// Copyright 2001, Jonathan Bachrach.  See file TERMS.

import java.io.*;
import java.util.*;
import antlr.*;

/**
 * Pattern Variable constraint on matching
 * @author  Jonathan Bachrach
 * @author  Keith Playford
 * @version 0.9, 03OCT00
 */
public abstract class BaseSyntaxConstraint {
    private Class _fragmentClass = null;
    public String   getFragmentClassName () { 
	return getFragmentClass().getName();
    }
    public Class   getFragmentClass() { 
	try {
	    return Class.forName("SequenceFragment"); 
	} catch (ClassNotFoundException e) {
	    Fragment.die("SequenceFragment not found");
	    return getClass(); // never reaches here
	}
    }
    public Class getFragmentClass (String name) {
	try {
	    return Class.forName(name); 
	} catch (ClassNotFoundException e) {
	    return getFragmentClass();
	}
    }
    private Boolean _doesMatchMultipleFragments = null;
    public boolean getDoesMatchMultipleFragments() {
	if (_doesMatchMultipleFragments == null) {
	    Class c = getFragmentClass();
	    while (true) {
		String name = c == null ? "java.lang.Object" : c.getName();
		if (name.equals("CompoundFragment")) {
		    _doesMatchMultipleFragments = Boolean.TRUE;
		    break;
		} else if (name.equals("java.lang.Object")) {
		    _doesMatchMultipleFragments = Boolean.FALSE;
		    break;
		}
		c = c.getSuperclass();
	    };
	}
	return _doesMatchMultipleFragments.booleanValue();
    }
    public boolean isWildcardConstraint () {
	return false;
    }
}
