// Copyright 2001, Jonathan Bachrach.  See file TERMS.

import java.io.*;
import java.util.*;
import antlr.*;

/**
 * A compound fragment with brackets
 * @author  Jonathan Bachrach
 * @author  Keith Playford
 * @version 0.9, 03OCT00
 */
public class NestedFragment extends CompoundFragment {
    String left, right;
    Token leftToken, rightToken;
    FragmentList nestedFragments = FragmentList.FragmentNil;
    String nestedMatcherName     = null;
    String partMatcherName       = null;
    String strictPartMatcherName = null;

    public FragmentList getNestedFragments() { return nestedFragments; }
    public FragmentList getInsideFragments() { return getNestedFragments(); }
    public void setInsideFragments(FragmentList fs) { nestedFragments = fs; }

    public NestedFragment(FragmentList children) {
	this.nestedFragments = children;
    }

    public List tokens () {
	return List.list(leftToken)
	    .append(getInsideFragments().tokens())
	    .append(List.list(rightToken));
    }
	
    public Fragment expandTemplates () {
	getInsideFragments().expandTemplates();
	return this;
    } 
    public Fragment compileTemplate(int depth) {
	return super.compileTemplate(depth);
    }
    public Fragment markMacros (MacroExpander expander) { 
	setInsideFragments(getInsideFragments().markMacros(expander));
	return this;
    } 
    public Fragment expandMacros (MacroExpander expander, boolean isRecursive) { 
	getInsideFragments().expandMacros(expander, isRecursive);
	return this;
    } 

    public Expansion compilePatternElement 
	(FragmentList tail, IdentifierFragment nowVar, IdentifierFragment endVar) {
	// #{ SplitList nested    = ?nowVar.matchParens();
        //    FragmentList inside = nested.getInside();
        //    FragmentList after  = nested.getAfter();
	//    ?nestedCode; 
	//    ?afterCode }
	String nestedMatcher         = nestedMatcherName;
	IdentifierFragment tmpVar    = IdentifierFragment.genSym("tmp");
	IdentifierFragment nestedVar = IdentifierFragment.genSym("ns");
	IdentifierFragment afterVar  = IdentifierFragment.genSym("fs");
	List parts                   = nestedFragments.splitAtCommas();
	LooseFragmentParser.trace("parts " + parts);
	Expansion nestedCode         = compileNestedPatternElements(parts, nestedVar);
	LooseFragmentParser.trace("after");
	Expansion afterCode          = tail.compilePatternElements(afterVar, endVar);
	return Template.processTemplate
	    (FragmentList.fnil
	     .fpush(Template.substituteIdentifier("SplitList"))
	     .fpush(tmpVar)
	     .fpush(Template.substituteIdentifier("="))
	     .fpush(nowVar)
	     .fpush(Template.substituteDot())
	     .fpush(Template.substituteIdentifier(nestedMatcher))
	     .fpush(Template.processParens(FragmentList.fnil))
	     .fpush(Template.substituteSemicolon())
	     .fpush(Template.substituteIdentifier("FragmentList"))
	     .fpush(nestedVar)
	     .fpush(Template.substituteIdentifier("="))
	     .fpush(tmpVar)
	     .fpush(Template.substituteDot())
	     .fpush(Template.substituteIdentifier("getInside"))
	     .fpush(Template.processParens(FragmentList.fnil))
	     .fpush(Template.substituteSemicolon())
	     .fpush(nestedCode)
	     .fpush(Template.substituteIdentifier("FragmentList"))
	     .fpush(afterVar)
	     .fpush(Template.substituteIdentifier("="))
	     .fpush(tmpVar)
	     .fpush(Template.substituteDot())
	     .fpush(Template.substituteIdentifier("getAfter"))
	     .fpush(Template.processParens(FragmentList.fnil))
	     .fpush(Template.substituteSemicolon())
	     .fpush(afterCode)
	     .freverse());
    }

    public Expansion compileNestedPatternElements
	(List parts, IdentifierFragment nowVar) {
	if (parts.size() == 1) {
	    FragmentList part = (FragmentList)(parts.getHead());
	    return part.compilePatternElements(nowVar, null);
	} else {
	    // #{ SplitList tmp = ?nowVar.?matcher();
	    //    FragmentList before = tmp.getBefore();
	    //    before.?elementMatcher();
	    //    FragmentList after  = tmp.getAfter();
	    //    ?afterCode }
	    String matcher
		= parts.size() == 2 ? partMatcherName : strictPartMatcherName;
	    IdentifierFragment tmpVar
		= IdentifierFragment.genSym("tmp");
	    IdentifierFragment beforeVar
		= IdentifierFragment.genSym("before");
	    IdentifierFragment afterVar
		= IdentifierFragment.genSym("after");
	    FragmentList part
		= (FragmentList)(parts.getHead());
	    Expansion beforeCode
		= part.compilePatternElements(beforeVar, null);
	    LooseFragmentParser.trace("before " + beforeCode);
	    Expansion afterCode
		= compileNestedPatternElements(parts.getTail(), afterVar);
	    LooseFragmentParser.trace("after " + afterCode);
	    return Template.processTemplate
		(FragmentList.fnil
		 .fpush(Template.substituteIdentifier("SplitList"))
		 .fpush(tmpVar)
		 .fpush(Template.substituteIdentifier("="))
		 .fpush(nowVar)
		 .fpush(Template.substituteDot())
		 .fpush(Template.substituteIdentifier(matcher))
		 .fpush(Template.processParens(FragmentList.fnil))
		 .fpush(Template.substituteSemicolon())
		 .fpush(Template.substituteIdentifier("FragmentList"))
		 .fpush(beforeVar)
		 .fpush(Template.substituteIdentifier("="))
		 .fpush(tmpVar)
		 .fpush(Template.substituteDot())
		 .fpush(Template.substituteIdentifier("getBefore"))
		 .fpush(Template.processParens(FragmentList.fnil))
		 .fpush(Template.substituteSemicolon())
		 .fpush(beforeCode)
		 .fpush(Template.substituteIdentifier("FragmentList"))
		 .fpush(afterVar)
		 .fpush(Template.substituteIdentifier("="))
		 .fpush(tmpVar)
		 .fpush(Template.substituteDot())
		 .fpush(Template.substituteIdentifier("getAfter"))
		 .fpush(Template.processParens(FragmentList.fnil))
		 .fpush(Template.substituteSemicolon())
		 .fpush(afterCode)
		 .freverse());
	}	    
    }

    void print(PrettyStream p) {
	p.println(left);
	nestedFragments.print(p);
	p.println(right);
    }

    void println(PrettyStream p) {
	p.print(left);
	nestedFragments.println(p);
	p.print(right);
    }

    void pprint(PrettyStream p, int depth, boolean isCode) {
	p.print(left);
	nestedFragments.pprint(p, depth, isCode);
	p.print(right);
    }
    void pprinte(PrettyStream p, int depth, boolean isLast, boolean isCode) { 
	pprint(p, depth, isCode);
	if (!isLast && !p.atBeginning) 
	    p.print(" " /* "X[" + getClass() + "]" */);
    }

    void lprint(PrettyStream p, int depth, boolean isCode) {
	lprint(p, left, leftToken.getLine(), depth);
	nestedFragments.lprint(p, depth + 1, isCode);
	lprint(p, right, rightToken.getLine(), depth);
    }
    void lprinte(PrettyStream p, int depth, boolean isLast, boolean isCode) { 
	lprint(p, depth, isCode);
	if (!isLast && !p.atBeginning) 
	    p.print(" " /* "X[" + getClass() + "]" */);
    }
    public String toString() { 
	return left + super.toString() + right;
    }
}

